from maya import cmds
import pymel.core as pm
import maya.mel as mm

"""To run this tool, save this entire script as python to your shelf

This script will bake your animation out on 2s and switch your your keys to stepped. 

To use, select the controls you want on 2s and run the script. This will either work on a selected range of your timeline,
otherwise it will use the whole timeline by default.

BakeOn2s v1.0 by Josh Janousky

JoshJanousky.com/downloads
"""

# Gets framerange
aPlayBackSliderPython = mm.eval('$tmpVar=$gPlayBackSlider')
keyRange = pm.timeControl( aPlayBackSliderPython, q=True, rangeArray=True)

# Freezes viewport
gMainPane = mm.eval( 'global string $gMainPane; $temp = $gMainPane;' )
pm.paneLayout(gMainPane, manage=False, e=1)

# key length = 1 when there is no highlight on timeline or only one frame is highlighted
kL = keyRange[1]-keyRange[0] 
if kL==1: # This would be if the time range is not highlighted

    # Bakes everything out on 2s
    sel=cmds.ls(sl=1)
    start=cmds.playbackOptions(q=1, min=1)
    end=cmds.playbackOptions(q=1, max=1)
    if sel:
        pm.refresh(su=True)
        cmds.bakeResults(sel, t=(start,end), sb=2, simulation=True, preserveOutsideKeys=True)
        pm.refresh(su=False)

        # Sets keys to stepped
        pm.keyTangent('graphEditor1FromOutliner', animation='objects', ott='step')

    cmds.warning("Your animation is now baked on 2s!")

# Unfreezes viewport
pm.paneLayout(gMainPane, manage=True, e=1)  

if kL != 1: # This would be if the range is selected
    
    # Bakes everything out on 2s
    sel=cmds.ls(sl=1)
    start=cmds.playbackOptions(q=1, min=1)
    end=cmds.playbackOptions(q=1, max=1)
    if sel:
        pm.refresh(su=True)
        cmds.bakeResults(sel, t=(keyRange[0],keyRange[1]), sb=2, simulation=True, preserveOutsideKeys=True)
        pm.refresh(su=False)

        # Sets keys to stepped
        pm.keyTangent('graphEditor1FromOutliner', animation='objects', ott='step')

    cmds.warning("Your animation is now baked on 2s!")

# Unfreezes viewport
pm.paneLayout(gMainPane, manage=True, e=1)  